function [t,info] = lpvrbperf(G,Ublksize,psi,gam)
% function [g,info] = simplerobperf(G,DeltaDim,Qconst)
%
% This function computes a bound on gain of Fu(G,Delta) where G is an
% LPV system and Delta a norm bounded uncertainty.  The code makes the following
% assumptions to study the use of parameter-varying IQCs:
%   1) G depends on a single parameter p
%   2) There are no rate bounds on the parameter p
%   3) Delta is norm-bounded and SISO (can be described by multiple IQCs)
%
% Inputs
%  G: An (nv+ne)-by-(nw+nd)-by-nmod array of state-space models that 
%      describe the dynamics of the LPV system G on a parameter grid of 
%      nmod points.  nv=nw are the dimensions of the delta (=DeltaDim)
%      and nd/ne are the dimensions of the disturbance/error signals
%      of the resulting interconnection Fu(G,Delta).
%  DeltaDim: Integer describing the dimension of the delayed signal.
%
% Outputs
%  g: Bound on the gain of Fu(G,Delta)
%  info:  Structure with information regarding solution of dissipation
%         inequality. Structure includes the fields:
%    -P:  Symmetric matrix used to construct the storage function.
%    -lambda: scaling coefficient
%


%% Dimensions
szG = size(G);
if numel(szG) == 2
    szG = [szG 1];
end
nmod = szG(3); % # of models in the G array.

szU = sum(Ublksize,1);
nw = szU(1);   % # of outputs of Delta
nv = szU(2);   % # of inputs to Delta

nd = szG(2)-nw;   % # of disturbances
ne = szG(1)-nv;   % # of errors


% Construct extended system Pext including dynamics of plant G and
% all auxiliary systems Psi. The I/O relationship for Pext is:
%     [y1; ...; yNiqc; e] = Pext*[z1; ...; zNblk; d]
% where zi are the outputs of uncertainty i and yj are the outputs
% of the auxiliary system psi for the j^th IQC.
vidx = cell(1,1);
zidx = cell(1,1);

zptr = 0;
vptr = 0;
AllPsi = [];
AllM = [];
yidx = [];
yptr = 0;
GIidx = [];
setlmis([])

% Grab indices associated with i^th uncertaint I/O channels

zidx = [zptr+1:zptr+Ublksize(1)]';
zptr = zptr + Ublksize(1);
vidx = [vptr+1:vptr+Ublksize(2)]';
vptr = vptr + Ublksize(2);

% Grab IQCs for the i1^th uncertainty and stack together
AllPsi = vertcat(psi{:});
    
% Store indices of v/z associated with I/O of i1^th uncertainty
GIidx = [GIidx; vidx; szG(1)+zidx ];

% Store output indices associated with all iqcs
niqcs = numel(psi);
scl = blkdiag(1/gam*eye(nv),eye(nw));
AllPsi = AllPsi*scl;
for i2=1:niqcs
    [lam(i2),nlam(i2)] = lmivar(1,[1 1]);
%     [~,~,xlam{i2}] = lmivar(3,nlam(i2)*eye(Ublksize(1)));
    szyi = size(psi{i2},1);
    yidx = [yidx; {yptr + (1:szyi)}];
    yptr = yptr+szyi;
end

ny = yptr; % # of y dimensions

% Append indices for error output channels
GIidx = [GIidx; [szG(1)-ne+1:szG(1)]'];
AllPsi = blkdiag(AllPsi,ss(eye(ne)));
AllPsi = repsys(AllPsi,[1 1 nmod]);
GI = [G;repmat(eye(szG(2)),[1 1 nmod])];
GI = GI(GIidx,:,:);
Pext = AllPsi*GI;

% Partition state matrices of Pext based on inputs [z;d] and outputs [y;e]
[A,B,C,D] = ssdata(Pext);
nx = size(A,1);
Ce = C(end-ne+1:end,:,:);
De = D(end-ne+1:end,:,:);


%%
% Initialize LMI and define variables
P = lmivar(1,[nx 1]);

gsq = gam^2;
% gsqI = blkdiag(zeros(nx+nw),gsq*eye(nd)); %for performance block
gsqI = blkdiag(zeros(nx+nw),eye(nd));
M = blkdiag(eye(Ublksize(1)),-eye(Ublksize(2))); %for IQC block
gsqinv = gsq^(-1);

%%
% Specify LMIs

% LMI for Dissipation Inequality
cnt = 1;
for k = 1:nmod
    Ak = A(:,:,k);
    Bk = B(:,:,k);
    Cek = Ce(:,:,k);
    Dek = De(:,:,k);
    
    % Terms involving plant and error signals
    lmiterm([cnt 1 1 P],[eye(nx); zeros(nw+nd,nx)],[Ak Bk],'s');
    lmiterm([-cnt 1 1 0],gsqI);
    lmiterm([cnt 1 1 0],gsqinv*[Cek Dek]'*[Cek Dek]);
    
    
    % Terms involving IQC for Delta
    for i1=1:niqcs
        Cyi = C(yidx{i1},:,k);
        Dyi = D(yidx{i1},:,k);
%         Mi = lmivar(3,blkdiag(xlam{i1},xlam{i1}));
        lmiterm([cnt 1 1 lam(i1)],1,[Cyi Dyi]'*M*[Cyi Dyi]);
    end
    
    cnt = cnt +1;
end

% LMI to enforce P>=0 if there are more than 1 IQC
if niqcs == 1
    lmiterm([-cnt 1 1 P],1,1);
    cnt  = cnt+1;
end

% LMI to enforce scalings to be non-negative
for j=1:niqcs
    lmiterm([-cnt 1 1 lam(j)],1,1);
    cnt = cnt + 1;
end

%% Solve SDP: min gamsq subject to LMI constraints
lmisys = getlmis;

opt = [0 0 0 0 1];
[tmin,xopt] = feasp(lmisys,opt);

%% Store solution
t = inf;
info.P = [];
info.lam = [];
if ~isempty(xopt)
    t = tmin;
    info.P = dec2mat(lmisys,xopt,P);
    lamopt = zeros(niqcs,1);
    for j=1:niqcs
        lamopt(j) = dec2mat(lmisys,xopt,lam(j));
    end
    info.lam = lamopt;
end
